#  python-holidays
#  ---------------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: dr-prodigy <maurizio.montel@gmail.com> (c) 2017-2022
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/dr-prodigy/python-holidays
#  License: MIT (see LICENSE file)

from .angola import AO, AGO, Angola
from .argentina import AR, ARG, Argentina
from .armenia import AM, ARM, Armenia
from .aruba import AW, ABW, Aruba
from .australia import AU, AUS, Australia
from .austria import AT, AUT, Austria
from .azerbaijan import AZ, AZE, Azerbaijan
from .bangladesh import BD, BGD, Bangladesh
from .belarus import BY, BLR, Belarus
from .belgium import BE, BEL, Belgium
from .bolivia import BO, BOL, Bolivia
from .bosnia_and_herzegovina import BA, BIH, BosniaAndHerzegovina
from .botswana import BW, BWA, Botswana
from .brazil import BR, BRA, Brazil
from .bulgaria import BG, BLG, Bulgaria
from .burundi import BI, BDI, Burundi
from .canada import CA, CAN, Canada
from .chile import CL, CHL, Chile
from .china import CN, CHN, China
from .colombia import CO, COL, Colombia
from .croatia import HR, HRV, Croatia
from .cuba import CU, CUB, Cuba
from .curacao import CW, CUW, Curacao
from .cyprus import CY, CYP, Cyprus
from .czechia import CZ, CZE, Czechia
from .denmark import DK, DNK, Denmark
from .djibouti import DJ, DJI, Djibouti
from .dominican_republic import DO, DOM, DominicanRepublic
from .egypt import EG, EGY, Egypt
from .estonia import EE, EST, Estonia
from .eswatini import SZ, SZW, Eswatini, Swaziland
from .ethiopia import ET, ETH, Ethiopia
from .finland import FI, FIN, Finland
from .france import FR, FRA, France
from .georgia import GE, GEO, Georgia
from .germany import DE, DEU, Germany
from .greece import GR, GRC, Greece
from .honduras import HN, HND, Honduras
from .hongkong import HK, HKG, HongKong
from .hungary import HU, HUN, Hungary
from .iceland import IS, ISL, Iceland
from .india import IN, IND, India
from .indonesia import ID, IDN, Indonesia
from .ireland import IE, IRL, Ireland
from .isle_of_man import IM, IMN, IsleOfMan
from .israel import IL, ISR, Israel
from .italy import IT, ITA, Italy
from .jamaica import JM, JAM, Jamaica
from .japan import JP, JPN, Japan
from .kazakhstan import KZ, KAZ, Kazakhstan
from .kenya import KE, KEN, Kenya
from .latvia import LV, LVA, Latvia
from .lesotho import LS, LSO, Lesotho
from .liechtenstein import LI, LIE, Liechtenstein
from .lithuania import LT, LTU, Lithuania
from .luxembourg import LU, LUX, Luxembourg
from .madagascar import MG, MDG, Madagascar
from .malawi import MW, MWI, Malawi
from .malaysia import MY, MYS, Malaysia
from .malta import MT, MLT, Malta
from .mexico import MX, MEX, Mexico
from .moldova import MD, MDA, Moldova
from .morocco import MA, MOR, Morocco
from .mozambique import MZ, MOZ, Mozambique
from .namibia import NA, NAM, Namibia
from .netherlands import NL, NLD, Netherlands
from .new_zealand import NZ, NZL, NewZealand
from .nicaragua import NI, NIC, Nicaragua
from .nigeria import NG, NGA, Nigeria
from .north_macedonia import MK, MKD, NorthMacedonia
from .norway import NO, NOR, Norway
from .pakistan import PK, PAK, Pakistan
from .paraguay import PY, PRY, Paraguay
from .peru import PE, PER, Peru
from .poland import PL, POL, Poland
from .portugal import PT, PRT, Portugal
from .romania import RO, ROU, Romania
from .russia import RU, RUS, Russia
from .saudi_arabia import SA, SAU, SaudiArabia
from .serbia import RS, SRB, Serbia
from .singapore import SG, SGP, Singapore
from .slovakia import SK, SVK, Slovakia
from .slovenia import SI, SVN, Slovenia
from .south_africa import ZA, ZAF, SouthAfrica
from .south_korea import KR, KOR, Korea, SouthKorea
from .spain import ES, ESP, Spain
from .sweden import SE, SWE, Sweden
from .switzerland import CH, CHE, Switzerland
from .taiwan import TW, TWN, Taiwan
from .tunisia import TN, TUN, Tunisia
from .turkey import TR, TUR, Turkey
from .ukraine import UA, UKR, Ukraine
from .united_arab_emirates import AE, ARE, UnitedArabEmirates
from .united_kingdom import GB, UK, GBR, UnitedKingdom
from .united_states import US, USA, UnitedStates
from .uruguay import UY, URY, Uruguay
from .uzbekistan import UZ, UZB, Uzbekistan
from .venezuela import VE, VEN, Venezuela
from .vietnam import VN, VNM, Vietnam
from .zambia import ZM, ZMB, Zambia
from .zimbabwe import ZW, ZWE, Zimbabwe
